@@ -38,11 +38,20 @@ namespace facebook::hbt {
 
 struct FdWrapper {
   FdWrapper(const FdWrapper&) = delete;
-  FdWrapper(FdWrapper&&) = delete;
+  FdWrapper(FdWrapper&& other) noexcept {
+    fd = other.fd;
+    close_on_destruction = other.close_on_destruction;
+
+    other.fd = -1;
+    other.close_on_destruction = false;
+  }
 
   explicit FdWrapper() {}
 
-  explicit FdWrapper(int fd) : fd{fd} {}
+  explicit FdWrapper(int fd) : FdWrapper(fd, false) {}
+
+  explicit FdWrapper(int fd, bool close_on_destruction)
+      : fd{fd}, close_on_destruction(close_on_destruction) {}
 
   explicit FdWrapper(const std::string& path, int flags = O_RDONLY) {
     fd = ::open(path.c_str(), flags);
@@ -51,7 +60,7 @@ struct FdWrapper {
     close_on_destruction = true;
   }
 
-  __u64 getInode() {
+  ino_t getInode() {
     struct stat s = {};
     int err = ::fstat(fd, &s);
     HBT_THROW_SYSTEM_IF(0 != err, errno)
@@ -459,6 +468,40 @@ inline TimeStamp rdtsc() {
 #endif
 }
 
+inline uint64_t rdpmc(int idx) {
+  uint64_t val;
+
+#if defined(__x86_64__)
+  val = __rdpmc(idx);
+#elif defined(__aarch64__)
+  /* TODO: Test and verify this */
+  switch (idx) {
+    case 0:
+      __asm__ volatile("mrs %0, pmevcntr0_el0" : "=r"(val));
+      break;
+    case 1:
+      __asm__ volatile("mrs %0, pmevcntr1_el0" : "=r"(val));
+      break;
+    case 2:
+      __asm__ volatile("mrs %0, pmevcntr2_el0" : "=r"(val));
+      break;
+    case 3:
+      __asm__ volatile("mrs %0, pmevcntr3_el0" : "=r"(val));
+      break;
+    case 4:
+      __asm__ volatile("mrs %0, pmevcntr4_el0" : "=r"(val));
+      break;
+    case 5:
+      __asm__ volatile("mrs %0, pmevcntr5_el0" : "=r"(val));
+      break;
+    default:
+      val = 0ULL;
+      break;
+  }
+#endif
+  return val;
+}
+
 #if defined(__i386__) || defined(__x86_64__)
 // Store CPU ID in <cpu> variable.
 inline TimeStamp rdtscp(CpuId& cpu) {
