@@ -103,7 +103,7 @@ class TraceCollector {
   explicit TraceCollector(const CpuSet& mon_cpus)
       : state_{State::Closed}, mon_cpus_{mon_cpus}, dummy_gen_{0} {
     dummy_gen_.open();
-    dummy_gen_.enable();
+    dummy_gen_.enable(true);
   }
 
   auto getMonCpus() const {
@@ -174,6 +174,11 @@ class TraceCollector {
     sync_();
   }
 
+  void openForCpu(CpuId cpu) {
+    HBT_THROW_EINVAL()
+        << "openForCpu() << is not implemented for TraceCollector";
+  }
+
   // Enabled implies open.
   bool isOpen() const {
     std::lock_guard<std::mutex> slices_lock{slices_mutex_};
@@ -181,6 +186,12 @@ class TraceCollector {
     return state_ == State::Open || state_ == State::Enabled;
   }
 
+  bool isOpenOnCpu(CpuId cpu) const {
+    HBT_THROW_EINVAL()
+        << "isOpenOnCpu() << is not implemented for TraceCollector";
+    return false;
+  }
+
   void close() {
     std::lock_guard<std::mutex> slices_lock{slices_mutex_};
     std::lock_guard<std::mutex> counts_lock{counts_mutex_};
@@ -188,19 +199,35 @@ class TraceCollector {
     sync_();
   }
 
-  void enable() {
+  void closeForCpu(CpuId cpu) {
+    HBT_THROW_EINVAL() << "closeForCpu() is not implemented for TraceCollector";
+  }
+
+  void enable(bool reset) {
     std::lock_guard<std::mutex> slices_lock{slices_mutex_};
     std::lock_guard<std::mutex> counts_lock{counts_mutex_};
     state_ = State::Enabled;
+    reset_ = reset;
     sync_();
   }
 
+  void enableForCpu(bool reset, CpuId cpu) {
+    HBT_THROW_EINVAL()
+        << "enableForCpu() is not implemented for TraceCollector";
+  }
+
   bool isEnabled() const {
     std::lock_guard<std::mutex> slices_lock{slices_mutex_};
     std::lock_guard<std::mutex> counts_lock{counts_mutex_};
     return state_ == State::Enabled;
   }
 
+  bool isEnabledOnCpu(CpuId cpu) const {
+    HBT_THROW_EINVAL()
+        << "isEnabledOnCpu() << is not implemented for TraceCollector";
+    return false;
+  }
+
   void disable() {
     std::lock_guard<std::mutex> slices_lock{slices_mutex_};
     std::lock_guard<std::mutex> counts_lock{counts_mutex_};
@@ -208,6 +235,15 @@ class TraceCollector {
     sync_();
   }
 
+  void disableForCpu(CpuId cpu) {
+    HBT_THROW_EINVAL()
+        << "disableForCpu() is not implemented for TraceCollector";
+  }
+
+  std::set<CpuId> listCpus() const {
+    return getMonCpus().asSet();
+  }
+
   /// Read events in PerCpuCountSampleGenerator.
   /// This is possible because perf_events opened in sample
   /// mode can be read as regular events.
@@ -598,6 +634,8 @@ class TraceCollector {
   std::unique_ptr<std::thread> counts_accum_thread_;
   std::atomic<bool> do_accum_threads_ = false;
 
+  bool reset_ = true;
+
   void sync_();
 
   size_t numCountGenerators_() const {
