@@ -9,6 +9,7 @@
 #include <sys/file.h>
 #include <unistd.h>
 #include <algorithm>
+#include <memory>
 #include <optional>
 #include "hbt/src/perf_event/BPerfEventsGroup.h"
 #include "hbt/src/perf_event/BuiltinMetrics.h"
@@ -16,9 +17,6 @@
 
 namespace facebook::hbt::perf_event {
 
-constexpr auto kAttrMapVersion = 1;
-constexpr auto kAttrMapSize = 16;
-
 static inline __u32 bpf_link_get_id(int fd) {
   struct bpf_link_info link_info = {
       .id = 0,
@@ -61,12 +59,14 @@ static inline __u32 bpf_prog_get_id(int fd) {
 
 /// name: Path of ebpf map
 BPerfEventsGroup::BPerfEventsGroup(
-    const std::string& name,
     const EventConfs& confs,
-    int cgroup_update_level)
-    : name_(name), confs_(confs), cgroup_update_level_(cgroup_update_level) {
-  HBT_ARG_CHECK_LE(name.length(), BPERF_METRIC_NAME_SIZE)
-      << "bpf map name is too long, max size is " << BPERF_METRIC_NAME_SIZE;
+    int cgroup_update_level,
+    bool support_per_thread,
+    const std::string& pin_name)
+    : confs_(confs),
+      cgroup_update_level_(cgroup_update_level),
+      per_thread_(support_per_thread),
+      pin_name_(pin_name) {
   for (const auto& conf : confs_) {
     struct perf_event_attr attr = {
         .size = sizeof(attr),
@@ -82,18 +82,20 @@ BPerfEventsGroup::BPerfEventsGroup(
 }
 
 BPerfEventsGroup::BPerfEventsGroup(
-    const std::string& name,
     const MetricDesc& metric,
     const PmuDeviceManager& pmu_manager,
-    int cgroup_update_level)
+    int cgroup_update_level,
+    bool support_per_thread,
+    const std::string& pin_name)
     : BPerfEventsGroup(
-          name,
           metric.makeNoCpuTopologyConfs(pmu_manager),
-          cgroup_update_level) {}
-inline auto mapFdWrapperPtrIntoInode(
+          cgroup_update_level,
+          support_per_thread,
+          pin_name) {}
+inline ino_t mapFdWrapperPtrIntoInode(
     const std::shared_ptr<FdWrapper>& fd_wrapper) {
   if (fd_wrapper == nullptr) {
-    return 0ull;
+    return static_cast<ino_t>(0);
   }
   return fd_wrapper->getInode();
 }
@@ -102,23 +104,6 @@ BPerfEventsGroup::~BPerfEventsGroup() {
   this->close();
 }
 
-std::string BPerfEventsGroup::attrMapPath() {
-  std::stringstream ss;
-
-  ss << "/sys/fs/bpf/bperf_attr_map_v";
-  ss << std::setw(3) << std::setfill('0') << kAttrMapVersion;
-  return ss.str();
-}
-
-// TODO: deprecate attr map?
-bperf_attr_map_key::bperf_attr_map_key(std::string n, int s) : size(s) {
-  ::memset(name, 0, BPERF_METRIC_NAME_SIZE);
-  ::memcpy(
-      name,
-      n.c_str(),
-      std::min(static_cast<int>(n.size()), BPERF_METRIC_NAME_SIZE - 1));
-}
-
 size_t BPerfEventsGroup::getNumEvents() const {
   return attrs_.size();
 }
@@ -213,13 +198,40 @@ void BPerfEventsGroup::close() {
     ::close(fd);
     fd = -1;
   }
+  // Close the perf event fds before destroying the links for per thread
+  // monitoring. This will help the reader detect the lead has exited.
+  ::bpf_link__destroy(register_thread_link_);
+  register_thread_link_ = nullptr;
+  ::bpf_link__destroy(unregister_thread_link_);
+  unregister_thread_link_ = nullptr;
+  ::bpf_link__destroy(pmu_enable_exit_link_);
+  pmu_enable_exit_link_ = nullptr;
+  ::bpf_link__destroy(update_thread_link_);
+  update_thread_link_ = nullptr;
   opened_ = false;
 }
+
+std::string BPerfEventsGroup::perThreadArrayMapPath(const std::string& n) {
+  std::stringstream ss;
+
+  ss << "/sys/fs/bpf/bperf_per_thread_array_" << n;
+  return ss.str();
+}
+
+std::string BPerfEventsGroup::perThreadIndexMapPath(const std::string& n) {
+  std::stringstream ss;
+
+  ss << "/sys/fs/bpf/bperf_per_thread_index_" << n;
+  return ss.str();
+}
+
 bool BPerfEventsGroup::isOpen() const {
   return opened_;
 }
 
-[[nodiscard]] bool BPerfEventsGroup::enable() {
+[[nodiscard]] bool BPerfEventsGroup::enable(bool reset) {
+  HBT_ARG_CHECK_EQ(reset, false)
+      << "BPerfEventsGroup does not support resetting counters";
   if (enabled_) {
     HBT_LOG_WARNING() << "BPerfEventsGroup is already enabled.";
     return true;
@@ -236,8 +248,8 @@ bool BPerfEventsGroup::isOpen() const {
   HBT_DCHECK(leader_link_fd_ >= 0);
   // set proper offset_
   ::memset(offsets_, 0, sizeof(offsets_));
-  readGlobal(offsets_, true);
   enabled_ = true;
+  readGlobal(offsets_, true);
   return true;
 }
 
@@ -247,7 +259,7 @@ int BPerfEventsGroup::syncCpu_(__u32 cpu, int leader_fd) {
   DECLARE_LIBBPF_OPTS(
       bpf_test_run_opts,
       opts,
-      .ctx_in = NULL,
+      .ctx_in = nullptr,
       .ctx_size_in = 0,
       .flags = BPF_F_TEST_RUN_ON_CPU,
       .cpu = cpu,
@@ -255,7 +267,7 @@ int BPerfEventsGroup::syncCpu_(__u32 cpu, int leader_fd) {
   return ::bpf_prog_test_run_opts(leader_fd, &opts);
 }
 
-void BPerfEventsGroup::syncGlobal_() {
+void BPerfEventsGroup::syncGlobal_() const {
   int err;
 
   for (int cpu = 0; cpu < cpu_cnt_; cpu++) {
@@ -298,14 +310,29 @@ void bperf_attr_map_elem::loadFromSkelLink(
   }
 
   ::bpf_map__set_max_entries(skel->maps.events, cpu_cnt_ * event_cnt);
+
+  per_thread_data_size_ = sizeof(struct bperf_thread_data) +
+      sizeof(struct bperf_perf_event_data) * event_cnt;
+  /* roundup to 64 byte aligned */
+  per_thread_data_size_ = bperf_roundup(per_thread_data_size_, 64);
+  ::bpf_map__set_value_size(skel->maps.per_thread_data, per_thread_data_size_);
+
+  skel->bss->cpu_cnt = cpu_cnt_;
+  skel->rodata->event_cnt = event_cnt;
+  skel->bss->cgroup_update_level = cgroup_update_level_;
+
+  if (!per_thread_) {
+    bpf_program__set_autoload(skel->progs.bperf_register_thread, false);
+    bpf_program__set_autoload(skel->progs.bperf_unregister_thread, false);
+    bpf_program__set_autoload(skel->progs.bperf_update_thread, false);
+    bpf_program__set_autoload(skel->progs.bperf_pmu_enable_exit, false);
+    bpf_program__set_autoload(skel->progs.find_perf_events, false);
+  }
   if (err = bperf_leader_cgroup__load(skel); err) {
     HBT_LOG_ERROR() << "Failed to load skeleton.";
     return err;
   }
 
-  skel->bss->cpu_cnt = cpu_cnt_;
-  skel->bss->event_cnt = event_cnt;
-  skel->bss->cgroup_update_level = cgroup_update_level_;
   link = ::bpf_program__attach(skel->progs.bperf_on_sched_switch);
   if (!link) {
     HBT_LOG_ERROR() << "Failed to attach leader program";
@@ -327,12 +354,139 @@ void bperf_attr_map_elem::loadFromSkelLink(
     HBT_LOG_ERROR() << "Failed to load perf events";
   }
 
+  if (per_thread_) {
+    preparePerThreadBPerf_(skel);
+  }
 out:
   bperf_leader_cgroup__destroy(skel);
   ::bpf_link__destroy(link);
   return err;
 }
 
+int BPerfEventsGroup::pinThreadMaps_(bperf_leader_cgroup* skel) {
+  int err, map_fd;
+  auto path = BPerfEventsGroup::perThreadIndexMapPath(pin_name_);
+
+  map_fd = ::bpf_map__fd(skel->maps.per_thread_idx);
+  ::unlink(path.c_str());
+  if (err = ::bpf_obj_pin(map_fd, path.c_str()); err) {
+    HBT_LOG_WARNING() << "Someone pinned the map at " << path << ". "
+                      << "Error: " << toErrorCode(-err).message();
+    return -1;
+  }
+
+  path = BPerfEventsGroup::perThreadArrayMapPath(pin_name_);
+  map_fd = ::bpf_map__fd(skel->maps.per_thread_data);
+  ::unlink(path.c_str());
+  if (err = ::bpf_obj_pin(map_fd, path.c_str()); err) {
+    HBT_LOG_WARNING() << "Someone pinned the map at " << path << ". "
+                      << "Error: " << toErrorCode(-err).message();
+    return -1;
+  }
+  return 0;
+}
+
+int BPerfEventsGroup::preparePerThreadBPerf_(bperf_leader_cgroup* skel) {
+  char buf[per_thread_data_size_];
+  struct bperf_thread_metadata* metadata = (struct bperf_thread_metadata*)buf;
+  int err, fd;
+  int zero = 0;
+
+  static_assert(
+      sizeof(struct bperf_thread_metadata) <= sizeof(struct bperf_thread_data));
+
+  err = lookupPerfEvent_(skel);
+  if (err) {
+    HBT_LOG_ERROR() << "Failed to lookup perf events";
+    return err;
+  }
+
+  err = pinThreadMaps_(skel);
+  if (err != 0) {
+    HBT_LOG_ERROR() << "Failed to pin maps for per thread monitoring";
+    return err;
+  }
+
+  fd = ::bpf_map__fd(skel->maps.per_thread_data);
+  skel->bss->per_thread_data_id = bpf_map_get_id(fd);
+
+  register_thread_link_ =
+      ::bpf_program__attach(skel->progs.bperf_register_thread);
+  if (!register_thread_link_) {
+    HBT_LOG_ERROR() << "Failed to attach register_per_thread";
+    return -1;
+  }
+
+  unregister_thread_link_ =
+      ::bpf_program__attach(skel->progs.bperf_unregister_thread);
+  if (!unregister_thread_link_) {
+    HBT_LOG_ERROR() << "Failed to attach trace_bpf_mmap_close";
+    goto error_out;
+  }
+
+  update_thread_link_ = ::bpf_program__attach(skel->progs.bperf_update_thread);
+  if (!update_thread_link_) {
+    HBT_LOG_ERROR() << "Failed to attach bperf_update_thread";
+    goto error_out;
+  }
+
+  /* Initialize idx_list */
+  for (int i = 0; i < BPERF_MAX_GROUP_SIZE; i++) {
+    skel->bss->idx_list[i] = i;
+  }
+
+  /* Use the first element of array per_thread_data for meta data */
+  skel->bss->idx_list_first_free = 1;
+  memset(metadata, 0, per_thread_data_size_);
+  err = ::bpf_map__lookup_elem(
+      skel->maps.per_thread_data,
+      &zero,
+      sizeof(zero),
+      metadata,
+      per_thread_data_size_,
+      0);
+  if (err) {
+    goto error_out;
+  }
+
+  metadata->metadata_size = sizeof(struct bperf_thread_metadata);
+  metadata->thread_data_size = sizeof(struct bperf_thread_data);
+  metadata->event_data_size = sizeof(struct bperf_perf_event_data);
+  metadata->event_cnt = skel->rodata->event_cnt;
+  metadata->flags = 0;
+
+  err = ::bpf_map__update_elem(
+      skel->maps.per_thread_data,
+      &zero,
+      sizeof(zero),
+      metadata,
+      per_thread_data_size_,
+      BPF_ANY);
+  if (err) {
+    goto error_out;
+  }
+
+  pmu_enable_exit_link_ =
+      ::bpf_program__attach(skel->progs.bperf_pmu_enable_exit);
+  if (!pmu_enable_exit_link_) {
+    HBT_LOG_ERROR() << "Failed to attach bperf_pmu_enable_exit";
+    goto error_out;
+  }
+
+  return 0;
+
+error_out:
+  ::bpf_link__destroy(register_thread_link_);
+  register_thread_link_ = nullptr;
+  ::bpf_link__destroy(unregister_thread_link_);
+  unregister_thread_link_ = nullptr;
+  ::bpf_link__destroy(update_thread_link_);
+  update_thread_link_ = nullptr;
+  ::bpf_link__destroy(pmu_enable_exit_link_);
+  pmu_enable_exit_link_ = nullptr;
+  return -1;
+}
+
 [[nodiscard]] int BPerfEventsGroup::loadPerfEvent_(
     struct bperf_leader_cgroup* skel) {
   int err = 0, e;
@@ -357,23 +511,35 @@ out:
   return err;
 }
 
-int BPerfEventsGroup::read(
-    struct bpf_perf_event_value* output,
+std::vector<struct bpf_perf_event_value> BPerfEventsGroup::readFromBpf_(
     int fd,
-    __u64 id,
-    bool skip_offset) {
-  auto event_cnt = confs_.size();
+    __u64 id) const {
   std::vector<struct bpf_perf_event_value> values(
       (size_t)cpu_cnt_ * BPERF_MAX_GROUP_SIZE);
-
   if (!enabled_) {
-    return -1;
+    return {};
   }
 
   syncGlobal_();
   if (int ret = ::bpf_map_lookup_elem(fd, &id, values.data()); ret) {
     HBT_LOG_ERROR() << "cannot look up key " << id
                     << " from output map. Return value: " << ret;
+    return {};
+  }
+
+  return values;
+}
+
+int BPerfEventsGroup::read(
+    struct bpf_perf_event_value* output,
+    int fd,
+    __u64 id,
+    bool skip_offset) const {
+  auto event_cnt = confs_.size();
+
+  auto values = readFromBpf_(fd, id);
+
+  if (values.empty()) {
     return -1;
   }
 
@@ -395,13 +561,35 @@ int BPerfEventsGroup::read(
   return event_cnt;
 }
 
+int BPerfEventsGroup::readPerCpu(
+    std::map<
+        int,
+        std::array<struct bpf_perf_event_value, BPERF_MAX_GROUP_SIZE>>& output,
+    int fd,
+    __u64 id) const {
+  auto event_cnt = confs_.size();
+
+  auto values = readFromBpf_(fd, id);
+
+  for (size_t e = 0; e < event_cnt; e++) {
+    for (int c = 0; c < cpu_cnt_; c++) {
+      int idx = c * BPERF_MAX_GROUP_SIZE + e;
+      output[c][e].counter = values[idx].counter;
+      output[c][e].enabled = values[idx].enabled;
+      output[c][e].running = values[idx].running;
+    }
+  }
+
+  return event_cnt;
+}
+
 int BPerfEventsGroup::readGlobal(
     struct bpf_perf_event_value* output,
-    bool skip_offset) {
+    bool skip_offset) const {
   return read(output, global_output_fd_, 0, skip_offset);
 }
 
-bool BPerfEventsGroup::readGlobal(ReadValues& rv, bool skip_offset) {
+bool BPerfEventsGroup::readGlobal(ReadValues& rv, bool skip_offset) const {
   const auto num_events = rv.getNumEvents();
   HBT_ARG_CHECK_EQ(confs_.size(), rv.getNumEvents());
 
@@ -414,13 +602,30 @@ bool BPerfEventsGroup::readGlobal(ReadValues& rv, bool skip_offset) {
   }
 }
 
-int BPerfEventsGroup::readCgroup(
-    struct bpf_perf_event_value* output,
-    __u64 id) {
+int BPerfEventsGroup::readGlobalPerCpu(
+    std::
+        map<int, std::array<struct bpf_perf_event_value, BPERF_MAX_GROUP_SIZE>>&
+            output) const {
+  return readPerCpu(output, global_output_fd_, 0);
+}
+
+bool BPerfEventsGroup::readGlobalPerCpu(std::map<int, ReadValues>& rv) const {
+  std::map<int, std::array<struct bpf_perf_event_value, BPERF_MAX_GROUP_SIZE>>
+      output;
+  int numEvents = readGlobalPerCpu(output);
+  for (auto& [cpu, values] : output) {
+    rv.insert({cpu, ReadValues(numEvents)});
+    toReadValues(rv.at(cpu), values.data());
+  }
+  return numEvents > 0;
+}
+
+int BPerfEventsGroup::readCgroup(struct bpf_perf_event_value* output, __u64 id)
+    const {
   return read(output, cgroup_output_fd_, id, true /* skip_offset */);
 }
 
-bool BPerfEventsGroup::readCgroup(ReadValues& rv, __u64 id) {
+bool BPerfEventsGroup::readCgroup(ReadValues& rv, __u64 id) const {
   const auto num_events = rv.getNumEvents();
   HBT_ARG_CHECK_EQ(confs_.size(), rv.getNumEvents());
 
@@ -433,42 +638,8 @@ bool BPerfEventsGroup::readCgroup(ReadValues& rv, __u64 id) {
   }
 }
 
-// TODO replace flock with mkdirmutex:
-//   https://github.com/cdown/mkdirmutex
-[[nodiscard]] int BPerfEventsGroup::lockAttrMap_() {
-  auto path = BPerfEventsGroup::attrMapPath();
-  int map_fd;
-  int err = -1;
-
-  if (::access(path.c_str(), F_OK)) {
-    HBT_LOG_INFO() << "Creating " << path;
-
-    map_fd = ::bpf_map_create(
-        BPF_MAP_TYPE_HASH,
-        nullptr,
-        sizeof(struct bperf_attr_map_key),
-        sizeof(struct bperf_attr_map_elem),
-        kAttrMapSize,
-        nullptr);
-    if (map_fd < 0) {
-      return -1;
-    }
-
-    if (err = ::bpf_obj_pin(map_fd, path.c_str()); err) {
-      HBT_LOG_WARNING() << "Someone pinned the map at " << path << ". "
-                        << "Error: " << toErrorCode(-err).message();
-      ::close(map_fd);
-    }
-  }
-  if (err) {
-    map_fd = ::bpf_obj_get(path.c_str());
-    if (map_fd < 0) {
-      return -1;
-    }
-  }
-  ::flock(map_fd, LOCK_EX);
-
-  return map_fd;
+EventConfs BPerfEventsGroup::getEventConfs() const {
+  return confs_;
 }
 
 void BPerfEventsGroup::toReadValues(
@@ -496,4 +667,50 @@ void BPerfEventsGroup::toReadValues(
   rv.t->time_running = (sum_time_running + num_events - 1) / num_events;
 }
 
+int BPerfEventsGroup::lookupPerfEvent_(struct bperf_leader_cgroup* skel) {
+  LIBBPF_OPTS(bpf_iter_attach_opts, opts);
+  union bpf_iter_link_info linfo;
+  struct bpf_link* link;
+  int iter_fd;
+  char buf[128];
+  int e, err;
+
+  for (e = 0; e < attrs_.size(); e++) {
+    for (int cpu = 0; cpu < cpu_cnt_; cpu++) {
+      int key = e * cpu_cnt_ + cpu;
+      int fd = pe_fds_[key];
+
+      err = ::bpf_map_update_elem(
+          ::bpf_map__fd(skel->maps.event_fd_to_idx), &fd, &key, BPF_ANY);
+      if (err) {
+        return err;
+      }
+    }
+  }
+
+  memset(&linfo, 0, sizeof(linfo));
+  linfo.task.pid = getpid();
+  opts.link_info = &linfo;
+  opts.link_info_len = sizeof(linfo);
+
+  skel->bss->leader_pid = linfo.task.pid;
+
+  link = bpf_program__attach_iter(skel->progs.find_perf_events, &opts);
+  if (!link) {
+    return -1;
+  }
+
+  iter_fd = bpf_iter_create(bpf_link__fd(link));
+  if (iter_fd < 0) {
+    err = -1;
+    goto out;
+  }
+  err = ::read(iter_fd, buf, sizeof(buf));
+
+out:
+  ::close(iter_fd);
+  ::bpf_link__destroy(link);
+  return err;
+}
+
 } // namespace facebook::hbt::perf_event
